package ejava.jdbc.advanced;


import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.lang.ClassNotFoundException;
import java.sql.PreparedStatement;
import java.util.Vector;
import java.util.prefs.Preferences;


/**
 * This class has the examples to use ReadOnly Scrollable ResultsSet
 * using Statement and PreparedStatement.
 */
public class ScrollableResultSetExample extends ConnectToDatabase
{
  
  /**
   * Constructor
   * @see
   */
  public ScrollableResultSetExample()
  {
    super();
  }
  
  
  
  /**
   * If it is not connected to database, connect to database and
   * create a statement and returns the Statement
   * @param scrollType     should be ResultSet.TYPE_FORWARD_ONLY
   * or  ResultSet.TYPE_SCROLL_INSENSITIVE
   * or  ResultSet.TYPE_SCROLL_SENSITIVE
   * @param updateType     should be ResultSet.CONCUR_READ_ONLY  or
   * ResultSet.CONCUR_UPDATABLE ;
   */
  protected Statement getStatement(int scrollType, int updateType)
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    Statement statement =
      connection.createStatement(scrollType, updateType);
    
    return statement;
  }
  
  /**
   * If it is not connected to database, connect to database and
   * create a preparestatement and return the statement
   * @param sqlString      queryString to make a statement
   * @param scrollType     should be ResultSet.TYPE_FORWARD_ONLY
   * or  ResultSet.TYPE_SCROLL_INSENSITIVE
   * or  ResultSet.TYPE_SCROLL_SENSITIVE
   * @param updateType     should be ResultSet.CONCUR_READ_ONLY  or
   * ResultSet.CONCUR_UPDATABLE ;
   */
  protected PreparedStatement getStatement(String sqlString,
    int scrollType, int updateType)
    throws SQLException,Exception
  {
    Connection connection = super.getConnection();
    PreparedStatement preparedStatement =
      connection.prepareStatement(sqlString, scrollType, updateType);
    
    return preparedStatement;
  }
  
  /**
   * this method executes the prepared statement and returns 
   * the resultSet after execution of a prepared statement.
   * The result set is scroll insensitive and read only.
   * only SELECT Statemet is applied here.
   */
  public ResultSet executeQueryScrollInsensitiveReadOnly(
    String sqlString,   Vector values)
    throws SQLException, Exception
  {
    PreparedStatement preparedStatement =
      getStatement(sqlString,
                   ResultSet.TYPE_SCROLL_INSENSITIVE,
                   ResultSet.CONCUR_READ_ONLY);
    
    for (int i = 1; i <= values.size(); i++) {
      preparedStatement.setObject(i, values.elementAt(i - 1));
    }
    
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = preparedStatement.executeQuery();
    
    return rs;
  }
  
  /**
   * this method executes the statement and returns the resultSet due to
   * execution of statement. only SELECT Statemet is applied here.
   * The result set is scroll insensitive and read only.
   */
  public ResultSet executeQueryScrollInsensitiveReadOnly(
    String sqlString)
    throws SQLException, Exception
  {
    Statement statement =
      getStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                   ResultSet.CONCUR_READ_ONLY);
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = statement.executeQuery(sqlString);
    
    return rs;
  }
  
  /**
   * this method executes the prepared statement and returns the 
   * resultSet due to execution of prepared statement. 
   * only SELECT Statemet is applied here.
   * The result set is scrollable forward only
   */
  public ResultSet executeQueryScrollForwardOnly(String sqlString,
                                                 Vector values)
    throws SQLException, Exception
  {
    PreparedStatement preparedStatement =
      getStatement(sqlString,ResultSet.TYPE_FORWARD_ONLY,
                   ResultSet.CONCUR_READ_ONLY);
    
    for (int i = 1; i <= values.size(); i++) {
      preparedStatement.setObject(i, values.elementAt(i - 1));
    }
    
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = preparedStatement.executeQuery();
    
    return rs;
  }
  
  /**
   * this method executes the statement and returns the resultSet due
   * to execution of Statement. only SELECT Statemet is applied here.
   * The result set is scrollable forward only.
   */
  public ResultSet executeQueryScrollForwardOnly(String sqlString)
    throws SQLException, Exception
  {
    Statement statement =
      getStatement(ResultSet.TYPE_FORWARD_ONLY,
                   ResultSet.CONCUR_READ_ONLY);
    
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = statement.executeQuery(sqlString);
    
    return rs;
  }
  
  /**
   * this method executes the prepared statement and returns the 
   * resultSet due to execution of PreparedStatement. 
   * only SELECT Statemet is applied here.
   * The result set is scroll sensitive and read only.
   */
  public ResultSet executeQueryScrollSensitiveReadOnly(
    String sqlString,       Vector values)
    throws SQLException, Exception
  {
    PreparedStatement preparedStatement =
      getStatement(sqlString,ResultSet.TYPE_SCROLL_SENSITIVE,
                   ResultSet.CONCUR_READ_ONLY);
    
    for (int i = 1; i <= values.size(); i++) {
      preparedStatement.setObject(i, values.elementAt(i - 1));
    }
    
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = preparedStatement.executeQuery();
    
    return rs;
  }
  
  /**
   * this method executes the  statement and returns the 
   * resultSet due to execution of Statement. 
   * only SELECT Statemet is applied here.
   * The result set is scroll sensitive and read only.
   */
  public ResultSet executeQueryScrollSensitiveReadOnly(
    String sqlString)
    throws SQLException, Exception
  {
    Statement statement =
      getStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
                   ResultSet.CONCUR_READ_ONLY);
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = statement.executeQuery(sqlString);
    
    return rs;
  }
  
  /**
   * prints ResultSet in the Forward Direction from current position;
   */
  public void printResultSetRowsInForwardDirection(ResultSet rs)
    throws SQLException {
    ResultSetMetaData rMetaData = rs.getMetaData();
    int nColumns = rMetaData.getColumnCount();
    
    for (int i = 1; i <= nColumns; i++) {
      System.out.print(rMetaData.getColumnName(i) + " : ");
    }
    
    System.out.println();
    
    while (rs.next()) {
      for (int i = 1; i < nColumns; i++) {
        System.out.print(rs.getString(i) + " : ");
      }
      System.out.println(rs.getString(nColumns));
    }
  }
  
  /**
   * prints ResultSet in the reverse direction from current position;
   */
  public void printResultSetRowsInReverseDirection(ResultSet rs)
    throws SQLException {
    int rsType = rs.getType();
    
    if (rsType == ResultSet.TYPE_FORWARD_ONLY
        || rsType == ResultSet.TYPE_SCROLL_SENSITIVE) {
      System.out
        .println("Error :ResultSet is not TYPE_SCROLL_INSENSITIVE");
      
      return;
    }
    
    ResultSetMetaData rMetaData = rs.getMetaData();
    int nColumns = rMetaData.getColumnCount();
    
    for (int i = 1; i < nColumns; i++) {
      System.out.print(rMetaData.getColumnName(i) + " : ");
    }
    
    System.out.println(rMetaData.getColumnName(nColumns));
    
    int fetchDirection = rs.getFetchDirection();
    
    if (fetchDirection == ResultSet.FETCH_FORWARD) {
      System.out
        .println("Result Set is  set to Fetch Forward Direction ");
    }
    
    rs.afterLast();
    
    while (rs.previous()) {
      for (int i = 1; i <= nColumns; i++) {
        System.out.print(rs.getObject(i) + " : ");
      }
      
      System.out.println(rs.getObject(nColumns));
    }
  }
  
  /**
   * prints nth row of result set;
   */
  public void printResultSetNthRow(ResultSet rs,int nthRow)
    throws SQLException
  {
    int rsType = rs.getType();
    
    if (rsType == ResultSet.TYPE_FORWARD_ONLY
        || rsType == ResultSet.TYPE_SCROLL_SENSITIVE) {
      System.out
        .println("Error :ResultSet is not TYPE_SCROLL_INSENSITIVE");
      
      return;
    }
    
    int fetchDirection = rs.getFetchDirection();
    
    if (fetchDirection != ResultSet.FETCH_FORWARD) {
      System.out
        .println("ResultSet is not set to Fetch Forward Direction");
      return;
    }
    // print metadata
    ResultSetMetaData rMetaData = rs.getMetaData();
    int nColumns = rMetaData.getColumnCount();
    
    rs.absolute(nthRow);
    
    int rowNumber = rs.getRow();
    
    System.out.println(" it is " + nthRow + "th Row in Result Set");
    
    if (rs.isAfterLast()) {
      System.out.println(" the Present Row is after last Row");
    }
    
    if (rs.isBeforeFirst()) {
      System.out.println(" The present Row is before first Row");
    }
    
    if (rs.isFirst()) {
      System.out.println("The present Row is Ist row");
    }
    
    if (rs.isLast()) {
      System.out.print(" The present Row is last Row");
    }
    // print metadata
    for (int i = 1; i < nColumns; i++) {
      System.out.print(rMetaData.getColumnName(i) + " : ");
    }
    
    System.out.println(rMetaData.getColumnName(nColumns));
    
    // print data in the row
    for (int i = 1; i < nColumns; i++) {
      System.out.print(rs.getObject(i) + " : ");
    }
    System.out.println(rs.getObject(nColumns));
    
  }
  
  /**
   * prints nth minus ith row of result set;
   */
  public void printNthMinusIthRow(ResultSet rs, int nthRow,
                                  int ithRow)
    throws SQLException
  {
    int rsType = rs.getType();
    
    if (rsType == ResultSet.TYPE_FORWARD_ONLY
        || rsType == ResultSet.TYPE_SCROLL_SENSITIVE) {
      System.out
        .println("Error :ResultSet is not TYPE_SCROLL_INSENSITIVE");
      
      return;
    }
    if (ithRow > nthRow) {
      System.out.println("i Should be less than n");
      
      return;
    }
    // get meta data
    ResultSetMetaData rMetaData = rs.getMetaData();
    int nColumns = rMetaData.getColumnCount();
    // move to nthRow
    rs.absolute(nthRow);
    // get row number
    int rowNumber = rs.getRow();
    
    System.out.println(" it is " + nthRow + "th Row in Result Set");
    // print meta data
    for (int i = 1; i < nColumns; i++) {
      System.out.print(rMetaData.getColumnName(i) + " : ");
    }
    System.out.print(rMetaData.getColumnName(nColumns) );
    
    // move to i rows down from current row
    rs.absolute(-ithRow);
    // print row
    for (int i = 1; i < nColumns; i++) {
      System.out.print(rs.getObject(i) + " : ");
    }
    System.out.println(rs.getObject(nColumns));
  }
  
  /**
   * Main execution method of scrollable result set examples
  */
  public static void main(String[] args)
  {
    System.out.println(" Scrollable Result Set Example ");    
    
    // Read preferences and create example object
    Preferences preferences 
        = Preferences.userNodeForPackage(
            ejava.jdbc.advanced.ConnectToDatabase.class);
            
    ScrollableResultSetExample scrollableResultSetExample 
        = new ScrollableResultSetExample();
        
    // Execute and print results for scroll insensitive
    // and read only result set example        
    try {
      System.out
        .println("Example to display the Scroll Insensitve"+
                   " only Result Set ");

      String sqlString 
            = preferences.get("SCROLLABLE_SQL_STATEMENT",null);

      ResultSet rs =
        scrollableResultSetExample
        .executeQueryScrollInsensitiveReadOnly(sqlString);
      
      scrollableResultSetExample
        .printResultSetRowsInForwardDirection(rs);
      scrollableResultSetExample
        .printResultSetRowsInReverseDirection(rs);
      scrollableResultSetExample.printResultSetNthRow(rs, 3);
      scrollableResultSetExample.printNthMinusIthRow(rs, 4, 2);
    }
    catch (ClassNotFoundException classNotFoundException) {
      String driverClass = preferences.get("DRIVER_CLASS",null);
      System.out.println("Driver Class :"+driverClass +" Exception:"
                           + classNotFoundException);
      System.exit(0);
    }
    catch (SQLException sqlException) {
      String dbURL = preferences.get("DATABASE_URL",null);
      System.out.println("Failed to connect db :"+dbURL+" Exception:"
                           + sqlException);
      System.exit(0);
    }
    catch (Exception exception) {
      System.out.println("Failed :" + " Exception :" + exception);
      System.exit(0);
    }
    
    // Execute and print results for scroll sensitive
    // and read only result set example        
    try {
      System.out
        .println("Example to display the Scroll Sensitve"+
                   " only Result Set ");
      
      String sqlString =
        preferences.get("SCROLLABLE_SQL_STATEMENT",null);

      ResultSet rs =
        scrollableResultSetExample
        .executeQueryScrollSensitiveReadOnly(sqlString);
      
      scrollableResultSetExample
        .printResultSetRowsInForwardDirection(rs);
      scrollableResultSetExample.printResultSetNthRow(rs, 3);
      scrollableResultSetExample.printNthMinusIthRow(rs, 4, 2);
    } catch (SQLException sqlException) {
      String dbURL = preferences.get("DATABASE_URL",null);
      System.out.println("Failed to connect db :" + dbURL
                           + " Exception :" + sqlException);
      System.exit(0);
    } catch (Exception exception) {
      System.out.println("Failed  :" + " Exception :" + exception);
      System.exit(0);
    }
    
    // Execute and print results for scroll sensitive
    // and read only result set from a PreparedStatement example      
    try {
      System.out
        .println("Scroll Insensitve ResultSet using a"+
                   " PreparedStatement ");

      String sqlString =
        preferences.get("SCROLLABLE_SQL_PREPARED_STATEMENT",null);
      Vector vector = new Vector();
      vector.addElement(preferences
           .get("SCROLLABLE_SQL_PREPARED_STATEMENT_VALUES",null));
      
      ResultSet rs =
        scrollableResultSetExample
        .executeQueryScrollInsensitiveReadOnly(sqlString, vector);
      
      scrollableResultSetExample
        .printResultSetRowsInForwardDirection(rs);
      scrollableResultSetExample
        .printResultSetRowsInReverseDirection(rs);
      scrollableResultSetExample.printResultSetNthRow(rs, 3);
      scrollableResultSetExample.printNthMinusIthRow(rs, 4, 2);
    } catch (SQLException sqlException) {
      String dbURL = (String) preferences.get("DATABASE_URL",null);
      System.out.println("Failed to connect db :" + dbURL +
         " Exception :" + sqlException);
      System.exit(0);
    } catch (Exception exception) {
      System.out.println("Failed :" + " Exception :" + exception);
      System.exit(0);
    }
    
    // Execute and print results for scroll forward result set     
     try {
      System.out.println("Result Example for Scroll Forward"+
                   " Only using Statement");
      
      String sqlString =
        preferences.get("SCROLLABLE_SQL_STATEMENT",null);

      ResultSet rs =
        scrollableResultSetExample
        .executeQueryScrollForwardOnly(sqlString);
      
      scrollableResultSetExample
        .printResultSetRowsInForwardDirection(rs);
    } catch (SQLException sqlException) {
      String dbURL = (String) preferences.get("DATABASE_URL",null);
      
      System.out.println("Failed to connect db :" + dbURL
                           + " Exception :" + sqlException);
      System.exit(0);
    } catch (Exception exception) {
      System.out.println("Failed :" + " Exception :" + exception);
      System.exit(0);
    }
    
    System.exit(0);
  }
  
}

