package ejava.jdbc.advanced;

import java.util.prefs.Preferences;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Savepoint;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.DatabaseMetaData;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.sql.PreparedStatement;

/**
 * This class shows how to use Updateable ResultSet objects
 */
public class UpdateableResultSetExample extends ConnectToDatabase
{

  /**
   * Constructor
   */
  public UpdateableResultSetExample()
  {
    super();
  }

  /**
   * if it is not connected to database, connect to database and
   * create a prepared statement and returns the statement
   * @param sqlString      queryString to make a statement
   * @param scrollType     should be ResultSet.TYPE_FORWARD_ONLY
   * or  ResultSet.TYPE_SCROLL_INSENSITIVE
   * or  ResultSet.TYPE_SCROLL_SENSITIVE
   * @param updateType     should be ResultSet.CONCUR_READ_ONLY  or
   * ResultSet.CONCUR_UPDATABLE ;
   */
  protected PreparedStatement getStatement(String sqlString,
    int scrollType,   int updateType)
    throws SQLException,    Exception
  {
    Connection connection = super.getConnection();
    connection.setAutoCommit(false);
    PreparedStatement preparedStatement 
      = connection.prepareStatement(sqlString, scrollType, updateType);

    return preparedStatement;
  }

  /**
   * if it is not connected to database, connect to database and
   * create a statement and return the statement
   * @param scrollType     should be ResultSet.TYPE_FORWARD_ONLY
   * or  ResultSet.TYPE_SCROLL_INSENSITIVE
   * or  ResultSet.TYPE_SCROLL_SENSITIVE
   * @param updateType     should be ResultSet.CONCUR_READ_ONLY  or
   * ResultSet.CONCUR_UPDATABLE ;
   */
  protected Statement getStatement(int scrollType, int updateType)
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    Statement statement 
        = connection.createStatement(scrollType, updateType);
    return statement;
  }

  /**
   * this method executes the statement and returns the 
   * resultSet due to execution of Statement.
   * only SELECT statemet is applicable here.
   */
  public ResultSet executeQueryScrollInsensitiveUpdateable(
                                                String sqlString)
    throws SQLException, Exception
  {
    Statement statement 
      = getStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                     ResultSet.CONCUR_UPDATABLE);

    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = statement.executeQuery(sqlString);

    return rs;
  }

  /**
   * this method executes the statement and returns the resultSet due
   * to execution of Statement. only SELECT Statement is applied here.
   */
  public ResultSet executeQueryScrollForwardOnlyUpdateable(
    String sqlString)
    throws SQLException, Exception
  {
    Statement statement
       = getStatement(ResultSet.TYPE_FORWARD_ONLY,
                     ResultSet.CONCUR_UPDATABLE);
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = statement.executeQuery(sqlString);

    return rs;
  }

  /**
   * This meethod returns Scroll Sensitive Updateable ResultSet
   */
  public ResultSet executeQueryScrollSensitiveUpdateable(
                                                    String sqlString)
    throws SQLException, Exception
  {
    Statement statement
      = getStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
                     ResultSet.CONCUR_UPDATABLE);
    // Execute a SQL statement that returns a single ResultSet.
    ResultSet rs = statement.executeQuery(sqlString);

    return rs;
  }

  /**
   * insert a row into Orders table using Updateable ResultSet.
   */
  public void insertNewRowIntoOrdersTableUsingColumnNames(
    ResultSet rs)
    throws SQLException
  {
    rs.moveToInsertRow();
    rs.updateInt("CUSTOMER_ID_FK", 101);
    rs.updateInt("ORDER_ID", 5001);
    rs.updateDate("ORDER_DATE",  new Date(System.currentTimeMillis()));
    rs.updateString("SHIP_INSTRUCT", "deliver to office if not home");
    rs.updateDate("SHIP_DATE", new Date(System.currentTimeMillis()));
    rs.updateFloat("SHIP_WEIGHT", 22.0f);
    rs.updateFloat("SHIP_CHARGE", 44.56f);
    rs.updateDate("PAID_DATE", new Date(System.currentTimeMillis()));
    rs.insertRow();
  }

  /**
   * insert a Row into Orders table using Updateable ResultSet
   */
  public void insertNewRowIntoOrdersTableUsingColumnIndexes(
    ResultSet rs)
    throws SQLException
  {
    rs.moveToInsertRow();
    // In the table 1 Column index for ORDER_ID and 2 for
    // CUSTOMER_ID_FK and so on.
    rs.updateInt(1, 101);
    rs.updateInt(2, 5005);
    rs.updateDate(3, new Date(System.currentTimeMillis()));
    rs.updateString(4, "deliver to office if not home");
    rs.updateDate(5, new Date(System.currentTimeMillis()));
    rs.updateFloat(6, 22.0f);
    rs.updateFloat(7, 44.56f);
    rs.updateDate(8, new Date(System.currentTimeMillis()) );
    rs.insertRow();
  }

  /**
   * this method deletes nth Row from the result Set
   */
  public void deleteNthRow(ResultSet rs,int rowNum)
    throws SQLException
  {
    rs.absolute(rowNum);
    rs.deleteRow();
  }

  /**
   * this methods updates a value for the nthRow, ith Column .
   */
  public void updateNthRowIthColumnValue(ResultSet rs, int nThRow,
                                         int iThColumn, Object value)
    throws SQLException
  {
    ResultSetMetaData rMetaData = rs.getMetaData();
    int type = rMetaData.getColumnType(iThColumn);
    String colName = rMetaData.getColumnName(iThColumn);

    rs.absolute(nThRow);

    switch (type) {

      case Types.CHAR:

      case Types.VARCHAR:

      case Types.LONGVARCHAR:
        rs.updateString(colName, value.toString());
        break;

      case Types.BIT:
        rs.updateByte(colName,
           Byte.valueOf(value.toString()).byteValue());
        break;

      case Types.TINYINT:

      case Types.SMALLINT:

      case Types.INTEGER:
        rs.updateInt(colName,
           Integer.valueOf(value.toString()).intValue());
        break;

      case Types.BIGINT:
        rs.updateLong(colName,
           Long.valueOf(value.toString()).longValue());
        break;

      case Types.FLOAT:
        rs.updateFloat(colName,
           Float.valueOf(value.toString()).floatValue());
        break;

      case Types.DOUBLE:
        rs.updateDouble(colName,
           Double.valueOf(value.toString()).doubleValue());
        break;

      case Types.DATE:
        rs.updateDate(colName, Date.valueOf(value.toString()));
        break;

      case Types.TIME:
        rs.updateTime(colName, Time.valueOf(value.toString()));
        break;

      case Types.TIMESTAMP:
        rs.updateTimestamp(colName,
           Timestamp.valueOf(value.toString()));
        break;

      default:
        rs.updateObject(colName, value);
        break;
    }
    rs.updateRow();
  }

  /**
   * cancel updates in a ResultSet
   */
  public void cancelUpdatesInResultSet(ResultSet rs)
    throws SQLException
  {
    rs.cancelRowUpdates();
  }

  /**
   * print ResultSet in the Forward Direction from current position;
   */
  public void printResultSetRowsInForwardDirection(ResultSet rs)
    throws SQLException
  {
    ResultSetMetaData rMetaData = rs.getMetaData();
    int nColumns = rMetaData.getColumnCount();
    // print metadata
    for (int i = 1; i < nColumns; i++) {
      System.out.print(rMetaData.getColumnName(i) + " : ");
    }

    System.out.println(rMetaData.getColumnName(nColumns));
    // print row
    while (rs.next()) {
      for (int i = 1; i < nColumns; i++) {
        System.out.print(rs.getString(i) + " : ");
      }

      System.out.println(rs.getString(nColumns));
    }
  }

  /**
   * Method
   * @param rs
   * @param nthRow
   *
   * @throws SQLException
   *
   * @see
   */
  public void printResultSetNthRow(ResultSet rs,int nthRow)
    throws SQLException
  {
    int rsType = rs.getType();
    if (rsType == ResultSet.TYPE_FORWARD_ONLY
        || rsType == ResultSet.TYPE_SCROLL_SENSITIVE) {
      System.out.println("Error-ResultSet not TYPE_SCROLL_INSENSITIVE");

      return;
    }

    ResultSetMetaData rMetaData = rs.getMetaData();
    int nColumns = rMetaData.getColumnCount();

    rs.absolute(nthRow);

    int rowNumber = rs.getRow();

    System.out.println(" it is " + nthRow + "th Row in Result Set");

    if (rs.isAfterLast()) {
      System.out.println(" the Present Row is after last Row");
    }

    if (rs.isBeforeFirst()) {
      System.out.println(" The present Row is before first Row");
    }

    if (rs.isFirst()) {
      System.out.println("The present Row is Ist row");
    }

    if (rs.isLast()) {
      System.out.print(" The present Row is last Row");
    }

    for (int i = 1; i < nColumns; i++) {
      System.out.print(rMetaData.getColumnName(i) + " : ");
    }

    System.out.println(rMetaData.getColumnName(nColumns));

    for (int i = 1; i < nColumns; i++) {
      System.out.print(rs.getObject(i) + " : ");
    }

    System.out.println(rs.getObject(nColumns));
  }

  /**
   * Method
   * @param rs
   * @param nthRow
   * @param ithRow
   *
   * @throws SQLException
   *
   * @see
   */
  public void printNthMinusIthRow(ResultSet rs, int nthRow,int ithRow)
    throws SQLException
  {
    int rsType = rs.getType();

    if (rsType == ResultSet.TYPE_FORWARD_ONLY
        || rsType == ResultSet.TYPE_SCROLL_SENSITIVE) {
      System.out.println("Error-ResultSet not TYPE_SCROLL_INSENSITIVE");
      return;
    }

    if (ithRow > nthRow) {
      System.out.println("i Should be less than n");
      return;
    }

    ResultSetMetaData rMetaData = rs.getMetaData();
    int nColumns = rMetaData.getColumnCount();
    rs.absolute(nthRow);
    int rowNumber = rs.getRow();
    System.out.println(" it is " + nthRow + "th Row in Result Set");

    for (int i = 1; i < nColumns; i++) {
      System.out.print(rMetaData.getColumnName(i) + " : ");
    }

    System.out.println(rMetaData.getColumnName(nColumns));

    for (int i = 1; i < nColumns; i++) {
      System.out.print(rs.getObject(i) + " : ");
    }

    System.out.println(rs.getObject(nColumns));
  }
  
  /**
   * Example snippet only of Savepoint usage
   */
  public void savePointExample(){
     Connection connection = null;
    
    try{
      // Get a connection as usual
      connection = getConnection();
      // Establish some SQL commands
      String blaSQLString = "INSERT bla INTO bladdybla";
      String fooSQLString = "INSERT foo INTO bar";
      // Establish some error conditions
      boolean someErrorCondition = false;
      boolean anotherErrorCondition = false;

      // Connection not in auto commit mode
      connection.setAutoCommit(false);

      // do some work...     
      
      Statement s1 = connection.createStatement();
      s1.executeUpdate( blaSQLString );
      
      // Set named savepoint
      Savepoint blaSP =  connection.setSavepoint("blaSavepoint");

      // do some more work...      
      
      // Rollback to first savepoint if neccessary
      if(someErrorCondition){
        System.out.println("Error in savepoint named "
                            + blaSP.getSavepointName());
        connection.rollback(blaSP);
      }      
      
      Statement s2 = connection.createStatement();
      s2.executeUpdate( fooSQLString );

      // Set unnamed savepoint
      Savepoint fooSP = connection.setSavepoint();

      // do even more work... 

      // Rollback to second savepoint if neccessary
      if(anotherErrorCondition){
        System.out.println("Error in savepoint with ID "
                            + fooSP.getSavepointId());
        connection.rollback(fooSP);
      }      
      
      connection.commit();
      s1.close();
      s2.close();
    }
    catch( Exception ex1){
      try{
        connection.rollback();
      }
      catch(SQLException ex2){
        // bla
      }
    }        
  }

  /**
   * Method main
   * @param args
   *
   * @see
   */
  public static void main(String[] args)
  {
    // Set preferences and create new example object
    Preferences preferences = Preferences.userNodeForPackage(
                              ejava.jdbc.advanced.ConnectToDatabase.class);
    String sqlString 
        = preferences.get("UPDATEABLE_SQL_STATEMENT",null);

    UpdateableResultSetExample example 
        = new UpdateableResultSetExample();

    try {
      // Show example of scroll insensitive updateable result set
      System.out.println("Scroll Insensitive Updateable ResultSet ");      
      // Execute query
      ResultSet rs 
        = example.executeQueryScrollInsensitiveUpdateable(sqlString);
      // print results
      example.printResultSetRowsInForwardDirection(rs);
      // insert new rows using column Names
      example.insertNewRowIntoOrdersTableUsingColumnNames(rs);
      // update a random row in the results
      example.updateNthRowIthColumnValue(rs, 1, 4, "Office delivery");


      // Show example of forward only updateable result set
      System.out.println("Forward Only Updateable ResultSet ");
      // Execute query
      rs = example.executeQueryScrollForwardOnlyUpdateable(sqlString);
      // insert using column indexes of the resultSet
      example.insertNewRowIntoOrdersTableUsingColumnIndexes(rs);
      // print results
      example.printResultSetRowsInForwardDirection(rs);
    }
    catch (ClassNotFoundException classNotFoundException) {
      String driverClass = preferences.get("DRIVER_CLASS",null);
      System.out.println("Driver Class :" + driverClass +
                           " Exception :"+ classNotFoundException);
      System.exit(0);
    }
    catch (SQLException sqlExcetpion) {
      String dbURL = preferences.get("DATABASE_URL",null);
      System.out.println("Failed to connect db :" + dbURL+
                           " Exception :" + sqlExcetpion);
      System.exit(0);
    }
    catch (Exception exception) {
      System.out.println("Failed to connect db :" + " Exception :"+
                           exception);
      System.exit(0);
    }

    System.exit(0);
  }

}

