package ejava.tcpip;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.InetAddress;
import java.util.Vector;
import java.util.prefs.Preferences;

/**
 * This is client class for the Server.
 * It registers the client receivers port to the server
 */
public class Client implements QueryServiceProtocol {
    private String               serverHost = "localhost";
    private int                  serverPort = 5555;
    private Socket               socket;
    private ObjectInputStream    remoteInputStream;
    private ObjectOutputStream   outputStream;
    private ClientReceiverServer clientReceiver;

    /**
     * Client takes serverHost name and portNumber on which
     * the server is running on host
     */
    public Client(String serverHost, int port) {
        if (serverHost != null) {
            serverHost = serverHost;
        } 

        if (port > -1) {
            serverPort = port;
        } 
    }

    /**
     * make connection to the server using the host and
     * port number
     */
    public void makeConnection() throws IOException {
        if (serverHost.indexOf("local") != -1) {
            serverHost = null;
        } 

        InetAddress serverAddress = InetAddress.getByName(serverHost);

        socket = new Socket(serverAddress, serverPort);
        outputStream = 
            new ObjectOutputStream(socket.getOutputStream());
    } 

    /**
     * to query for the customers who spend more than
     * a given value
     */
    public void queryForCustomersWhoSpendGreaterThan(float value) 
            throws IOException, Exception {

        // Write customer query to server socket
        outputStream.writeInt(CUSTOMERS_QUERY);
        outputStream.writeFloat(value);
        outputStream.flush();

        if (remoteInputStream == null) {
            remoteInputStream = 
                new ObjectInputStream(socket.getInputStream());
        } 

        // While still getting results from server
        boolean received = false;

        while (!received) {
            Vector vector = (Vector) remoteInputStream.readObject();

            // For each Vector element result, create a Customer object
            for (int i = 0; i < vector.size(); i++) {
                Customer customer = (Customer) vector.elementAt(i);

                System.out.println("First Name :" 
                                   + customer.getFirstName());
                System.out.println("Last Name :" 
                                   + customer.getLastName());
                System.out.println("Address1 Name :" 
                                   + customer.getAddress1());
                System.out.println("Address2 Name :" 
                                   + customer.getAddress2());
                System.out.println("City :" + customer.getCity());
                System.out.println("State :" + customer.getState());
                System.out.println("Zip :" + customer.getZip());
                System.out.println("Email Address :" 
                                   + customer.getEmailAddress());
                System.out.println("Phone :" 
                                   + customer.getPhoneNumber());
            } 

            received = true;
        } 
    } 

    /**
     * Register the receiver host and port, the server
     * sends the results of order query to the client
     * receiver
     */
    public void registerReceiverHostAndPort() throws IOException {

        // Read client socket info from preferences
	Preferences prefs = 
	    Preferences.userNodeForPackage(Client.class);
        String receiverHost = prefs.get("CLIENT_RECEIVER_HOST", null);
        int    receiverPort = prefs.getInt("CLIENT_RECEIVER_PORT", -1);

        if (receiverHost == null) {
            receiverHost = "localhost";
        } 

        // Create and start client callback socket
        if (clientReceiver == null) {
            clientReceiver = new ClientReceiverServer(receiverPort);

            clientReceiver.start();
        } 

        // Write client callback info to server
        outputStream.writeInt(REGISTER_RECEIVER);
        outputStream.writeObject(receiverHost);
        outputStream.writeInt(receiverPort);
        outputStream.flush();
    } 

    /**
     * Client asks for the Orders which are from a state,
     * the results are received by the client receiver,
     */
    public void queryForOrderFromState(String state) 
            throws IOException {
        outputStream.writeInt(ORDERS_QUERY);
        outputStream.writeObject(state);
        outputStream.flush();
    } 

    /**
     * client resources
     */
    public void clean() throws IOException {
        remoteInputStream.close();
        outputStream.close();
        socket.close();
    } 

    protected void finalize() throws Throwable {
        try {
            if (remoteInputStream != null) {
                remoteInputStream.close();
            } 

            if (outputStream != null) {
                outputStream.close();
            } 
        } catch (IOException ex) {}

        super.finalize();
    } 

    public static void main(String[] args) {

        Client c = null;

	// use preferences for Server class here to get port
        Preferences prefs = 
	    Preferences.userNodeForPackage(Server.class);

        String serverHost = prefs.get("SERVER_HOST", null);
        int    serverPort = prefs.getInt("SERVER_PORT", -1);

	try {
            // Create client, make server connection, issue customer query,
            // register callback, issue order query to receive callback
            c = new Client(serverHost, serverPort);

            c.makeConnection();
            c.queryForCustomersWhoSpendGreaterThan(100.00f);
            c.registerReceiverHostAndPort();
            c.queryForOrderFromState("VA");
            c.clean();
        } catch (Exception exception) {
            System.out.println(exception);
            exception.printStackTrace();

            try {
                c.clean();
            } catch (Exception exceptionClean) {
                exceptionClean.printStackTrace();
            } 

            System.exit(0);
        } 
    } 

}

