package ejava.webservices;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices
 * 
 * Type: SOAPServer
 * 
 */

import java.util.prefs.Preferences;

import java.net.*;
import java.io.*;
import javax.xml.soap.*;
import org.w3c.dom.*;
import java.util.*;

public class SOAPServer extends SOAPManager implements Runnable {

    private BufferedInputStream  inStream = null;
    private BufferedOutputStream outStream = null;
    private String host = "localhost";
    private String port = "9001";
    private String serverHost = "localhost";
    private String serverPort = "9000";

    /**
     * Create a SOAPServer with a client Socket. Get streams
     * and set server and client host and port values.
     */
    public SOAPServer(Socket client){
        try{            
            outStream 
                = new BufferedOutputStream(client.getOutputStream());
            inStream 
                = new BufferedInputStream(client.getInputStream());
                
            host = client.getInetAddress().getHostName();
            port = String.valueOf(client.getPort());
            serverHost = client.getLocalAddress().getHostName();
            serverPort = String.valueOf(client.getLocalPort());
        }
        catch(Exception e){
            printException(e);
        }
    }

    /**
     * Main thread for processing requests
     */
    public synchronized void run() {
        SOAPMessage response = null; // SOAP response

        try {                          
            // Parse and display request message
            String soapString = readMessage(inStream);
       
			// Process request and generate response
			response = process(soapString);
            
            // Display and send response message
            writeMessage(response, outStream);
        }
        catch (Exception e) {
            System.out.println("SOAP Server Exception: \n");
            printException(e);
        }
    }
    
        
    /**
     * Uses SOAPMessageHandler to parse message to and from
     * forms that the SOAP handlers understand and a SOAP message.
     * Also creates a new SOAPHandler based on message and delegates
     * processing to the handler.
     */
	private SOAPMessage process(String soapString) 
                                    throws SOAPException{	
        SOAPMessage response = createEmptyMessage();
        boolean mustUnderstand = true;
        try{           
            // Parse SOAP String request into HashMap
            HashMap inp = parseMessage(soapString);
                        
            // Get mustUnderstand value
            mustUnderstand 
                = ((Boolean) inp.get("mustUnderstand")).booleanValue();
            
            // Get the type of handler to invoke
            String classType = (String) inp.get("beeshirts:CLASS_NAME");
    		
    		// Based on request type, invoke appropriate handler
            SOAPHandler handler 
                = (SOAPHandler) Class.forName(classType).newInstance();
            
            // Then delgate call to concrete handler instance    
            HashMap outp = handler.process(inp);
            
            // Use HashMap response to create SOAP response
            response = createMessage(outp, host, port);
        }
        catch(Exception e){
            printException(e);
            if(mustUnderstand){
                createBodyFault(response, e, "UnsupportedHandler",
                                serverHost, serverPort);
            }
        }
    		
    	return response;
	}
	
    /**
     * Main method of SOAP Server. Reads preferences. Creates
     * new Server Socket listener. Accepts client requests.
     * Creates new thread to handle client request.
     */
	public static void main(String[] args) {
    	System.out.println("This is a SOAP Server.");
        
        // Get the preferences       
        Preferences preferences = Preferences.userNodeForPackage(
                                    ejava.webservices.SOAPServer.class);
 
        try {
            // Get server port
            int serverPort = preferences.getInt("SERVER_PORT", 9000);

            // Create a new socket server
        	ServerSocket server = new ServerSocket(serverPort);

	        /**
	         * Until user exists from the server using keyboard, CTRL-C
	         * or by shutting down from the server, it waits for
	         * clients to connect
	         */
	        while (true) {
	        	// Wait for client to connect
	            Socket client = server.accept();
				// Then create a new server handler object
	            SOAPServer soaper = new SOAPServer(client);
                // Create new thread for handler object
                Thread serverThread = new Thread(soaper);
				// Start the thread (see the run method)
	            serverThread.start();
        	} 
        } catch (Exception e) {
            System.out.println("SOAP Server Fatal Error: " + e);
            System.exit(0);
        } 		
	}
}
